#
# --------------------------------------------------------------------------- #
#                Protocol of Changes:
#
#  calc_protein_descriptors.py
#
#  24-feb-2015 (jm) created.
#
# *************************************************************************** #
#


__description__ =                                                       \
"""
calc_protein_descriptors.py

calculate antibody-specific descriptors for antibody structures

"""
__author__ = 'Johannes Maier  Schrodinger Inc.'
__date__ = '07/22/2016'

import argparse
import os
import sys
from pathlib import Path

from schrodinger.application.prime.packages import executeparallel as exc
from schrodinger.application.prime.packages import primeparser
from schrodinger.application.prime.packages.protein_descriptors import driver
from schrodinger.job import jobcontrol

# ------------------------------------------------------------------------  #

usage = """
$SCHRODINGER/run -FROM psp calc_protein_descriptors.py -i INPUTDIR -o OUTPUTDIR <options>
    Required arguments:
        -i  --inputdir   :   location (directory) of the input structures
        -o  --outputdir  :   output directory
    Optional arguments:
        -p  --pH         :   set the desired system pH (range 4.0 - 10.0)
                             (default: 7.0)
        -r  --custom_region:  specify custom regions as key/value parameters.
                              The values are expected to be valid asl strings 
                              and should describe consecutive regions within 
                              the protein.
        -h  --help       :   run this message

 The script will search for all valid structure files (.mae, .maegz, .pdb, .ent)
 in the specified input directory and calculates the the protein or antibody
 specific descriptors that are kept in the specified output directory.
 In the output directory, the following files / dirs are created:
    prepped: directory that stores the input structures but with the descriptor
             values stored as structure properties
    raw:     directory that stores the raw results of the descriptor calculations
             as a pickled python file
    pkl:     directory that stores all patch and residue specific data as pickles
    vis:     directory that stores the surface data files that may be
             imported/visualized in Maestro or PyMOL
    descriptors: directory that stores descriptor values as pickled data files

    protein_descriptors.csv :   csv-formatted output file (each structure is an entry)

"""

version = 'Calc_Protein_Descriptors 1.0.0'


def is_path_local_within_cwd(dir):
    """
    Return True if the given path is local path, and it's within the CWD.
    # OK: "output" and "./output"
    # NOT OK: "/absolute/path/to/output", "../to/output", "output/subdir"
    """
    dir = dir.rstrip('/')  # Ignore any trailing slashes
    return Path(dir) == Path(os.path.basename(dir))


class StoreDescriptorsInputDir(primeparser.StoreFile):
    """
    Register all files in input directory ...

    """

    def __call__(self, parser, namespace, dirname, option_string):
        """
        """
        filenames = os.listdir(dirname)
        for filename in filenames:
            super(StoreDescriptorsInputDir,
                  self).__call__(parser, namespace,
                                 os.path.join(dirname, filename), option_string)


# ==============================================


class CalcDescriptors(exc.ExecuteParallel):

    def __init__(self, script_fn):
        exc.ExecuteParallel.__init__(
            self,
            script_fn,
            usage, [],
            use_jobdj="never",
            logger=None,
            version=version)

    def setupParser(self):
        parser = primeparser.Parser(
            formatter_class=argparse.RawDescriptionHelpFormatter,
            description=usage)

        parser.add_argument(
            "-i",
            "--inputdir",
            required=True,
            action=StoreDescriptorsInputDir,  #'store',
            help="Specify directory containing input structures",
            dest='inputdir')
        parser.add_argument(
            "-o",
            "--outputdir",
            required=True,
            action='store',
            help="Specify directory for output",
            dest='outputdir')
        parser.add_argument(
            "-p",
            "--pH",
            action=primeparser.store_float(min=4.0, max=10.0),
            default=7.0,
            help="pH of the system",
            dest='pH')
        parser.add_argument(
            "-r",
            "--custom_region",
            action=primeparser.AddKVPToDict,
            dest='uregion',
            help="Define one or more custom-defined regions within the protein "
            "as a key/value format. The key specifies a unique name for the "
            "designated region, the value is expected to be a valid ASL "
            "string. Regions with invalid residue selections will be ignored.")

        return parser

    def preLaunchActions(self, opts, launch):
        if not is_path_local_within_cwd(opts.outputdir):
            raise self.parser.error(
                'Output directory must be specified as local path')
        return super().preLaunchActions(opts, launch)

    def runBackend(self, opts, istage):
        inputdir = os.path.dirname(opts.inputdir)
        outputdir = opts.outputdir
        pH = opts.pH
        ureg = opts.uregion

        if not inputdir or type(inputdir) != str:
            raise ValueError("input directory is not valid!")

        if not outputdir or type(outputdir) != str:
            raise ValueError("output directory is not valid!")

        if not ureg or type(ureg) != dict:
            ureg = {}

        backend = jobcontrol.get_backend()

        try:
            num_inp_files = driver.process_structures(inputdir, \
                    outputdir, pH=pH, custom_region=ureg)
        except Exception as e:
            print(str(e))
            print("protein descriptor calculation has failed.")
            raise
        else:
            if (num_inp_files == 0):
                print(f"No acceptable structure files found in {inputdir}")
            else:
                print("Processed %d files, output written to %s"\
                        % (num_inp_files, outputdir))

        # search for for output files to register
        if backend is not None:
            for dirpath, dirnames, filenames in os.walk(outputdir):
                for f in filenames:
                    backend.addOutputFile(os.path.join(dirpath, f))


# *************************************************************************** #
#                            main entry point                                 #
# *************************************************************************** #
if __name__ == "__main__":
    obj = CalcDescriptors(sys.argv[0])
    obj.launch(sys.argv[1:])
