#
# --------------------------------------------------------------------------- #
#                Protocol of Changes:
#
#  calc_protein_descriptors.py
#
#  24-feb-2015 (jm) created.
#
# *************************************************************************** #
#


__description__ =                                                       \
"""
calc_protein_descriptors.py

calculate antibody-specific descriptors for antibody structures

"""
__author__ = 'Johannes Maier  Schrodinger Inc.'
__date__ = '06/17/2019'

import argparse
import os
import sys

from schrodinger.application.prime.packages import executeparallel as exc
from schrodinger.application.prime.packages import primeparser
from schrodinger.application.prime.packages.protein_descriptors import seq_desc
from schrodinger.application.prime.packages.protein_descriptors import driver
from schrodinger.job import jobcontrol

# ------------------------------------------------------------------------  #

usage = f"""
$SCHRODINGER/run -FROM psp calc_sequence_descriptors.py -i INPUTDIR  <options>
    Required arguments:
        -i  --inputdir   :   location (directory) of the input structures

    Optional arguments:
        -d  --descriptors:   file with the names of the desired descriptors.
        -n  --prefix     :   prefix used in the output csv file
        -h  --help       :   run this message

 The script will search for fasta-formatted sequence files (.fst, .fasta)
 in the specified input directory and calculates the sequence-based
 descriptors to be stored in a csv-formatted output file named
 "sequence_descriptors.csv". Optionally, the user may issue a prefix that will
 be used for the output file. A user may also specify a newline-delimited
 descriptors file with the name of each descriptor they want calculated.
 Supported descriptors include: {', '.join(seq_desc.get_all_descriptor_names())}

"""

version = 'Calc_Sequence_Descriptors 1.0.0'


class StoreDescriptorsInputDir(primeparser.StoreFile):
    """
    Register all files in input directory ...

    """

    def __call__(self, parser, namespace, dirname, option_string):
        """
        """
        filenames = os.listdir(dirname)
        for filename in filenames:
            fn = os.path.join(dirname, filename)
            if not os.path.isfile(fn):
                continue

            super(StoreDescriptorsInputDir, self).__call__(
                parser, namespace, fn, option_string)


# ==============================================


class CalcSeqDescriptors(exc.ExecuteParallel):

    def __init__(self, script_fn):
        exc.ExecuteParallel.__init__(
            self,
            script_fn,
            usage, ['sequence_descriptors.csv'],
            use_jobdj="never",
            logger=None,
            version=version)

    def setupParser(self):
        parser = primeparser.Parser(
            formatter_class=argparse.RawDescriptionHelpFormatter,
            description=usage)

        parser.add_argument(
            "-i",
            "--inputdir",
            required=True,
            action=StoreDescriptorsInputDir,  #'store',
            help="Specify directory containing input structures",
            dest='inputdir')

        parser.add_argument(
            "-d",
            "--descriptors",
            default=None,
            help="Specify which descriptors to calculate",
            dest='descriptors')

        parser.add_argument(
            "-n",
            "--prefix",
            dest='prefix',
            default="",
            help='issue a prefix for the output file.')

        return parser

    def preLaunchActions(self, opts, launch):
        super().preLaunchActions(opts, launch)
        if opts.descriptors:
            launch.addInputFile(opts.descriptors)

    def runBackend(self, opts, istage):
        inputdir = os.path.dirname(opts.inputdir)

        pfx = opts.prefix

        if not inputdir or type(inputdir) != str:
            raise ValueError("input directory is not valid!")

        if not pfx or type(pfx) != str:
            pfx = ""

        backend = jobcontrol.get_backend()

        try:
            out_file = driver.process_sequences(
                inputdir, prefix=pfx, descriptors_file=opts.descriptors)
        except Exception as e:
            print(str(e))
            print("sequence descriptor calculation has failed.")
            raise

        backend.addOutputFile(out_file)


# *************************************************************************** #
#                            main entry point                                 #
# *************************************************************************** #
if __name__ == "__main__":
    obj = CalcSeqDescriptors(sys.argv[0])
    obj.launch(sys.argv[1:])
