"""
Script to extract FASTA sequence information from an Excel .xlsx file.

Copyright Schrodinger, LLC. All rights reserved.
"""
import argparse
from contextlib import contextmanager
import os
import openpyxl

from schrodinger.protein.alignment import ProteinAlignment
from schrodinger.protein.sequence import ProteinSequence
from schrodinger.utils import fileutils
from schrodinger.application.msv import seqio
from schrodinger.utils import log

logger = log.get_output_logger(__file__)
EXT = '.fasta'


@contextmanager
def xlsx_workbook(*args, **kwargs):
    """
    A context manager for loading an workbook using openpyxl.
    See `openpyxl.load_workbook` for arguments.
    """
    wb = openpyxl.load_workbook(*args, **kwargs)
    yield wb
    wb.close()


def write_unique_fasta(filename, protein_alignment):
    """
    Writes the protein alignment to file fname, ensuring that the created file
    has a unique name.

    :param filename: the preferred filename to use
    :type filename: str
    :param protein_alignment: the protein alignment to write
    :type protein_alignment: ProteinAlignment
    """

    if os.path.isfile(filename):
        filename = fileutils.get_next_filename(filename, midfix='-')
    seqio.FastaAlignmentWriter.write(
        protein_alignment, filename, use_unique_names=False)
    logger.info(f"Written {len(protein_alignment)} alignments to {filename}")


def get_table_sequences(ws, opts):
    """
    Extract the row sequences from the worksheet.

    :param ws: the worksheet
    :type ws: openpyxl.worksheet.worksheet.Worksheet
    :param opts: command line options, with some modifications
    :type opts: argparse.Namespace
    :return: Tuples of the sequence tuples encountered in the rows
    :rtype: Tuple[Tuple[ProteinAlignment]]
    """

    table_data = []
    chains = [ws[f'{col}{opts.chain}'].value for col in opts.cols]
    for row in opts.rows:
        name = ws[f'{opts.name}{row}'].value
        row_data = [
            ProteinSequence(ws[f'{col}{row}'].value, name=name, chain=chain)
            for chain, col in zip(chains, opts.cols)
        ]
        table_data.append(tuple(row_data))
    return tuple(table_data)


def convert_to_fasta(opts):
    """
    Converts the data in filename to one or more .fasta files.

    :param opts: command line options, with some modifications
    :type opts: argparse.Namespace
    """

    with xlsx_workbook(opts.input_file) as wb:
        if opts.sheet is None:
            ws = wb.active
        else:
            try:
                ws = wb.get_sheet_by_name(opts.sheet)
            except KeyError:
                logger.error(f'sheet {opts.sheet} is not a valid sheet name. '
                             f'Should be a name in {wb.get_sheet_names()}.')
                return

        logger.info(f'Reading {opts.input_file}.')
        table_sequences = get_table_sequences(ws, opts)

    if opts.separate:
        for row_sequences in table_sequences:
            base = row_sequences[0].name
            write_unique_fasta(f'{base}{EXT}', ProteinAlignment(row_sequences))
        return

    fname = f'{fileutils.get_basename(opts.input_file)}{EXT}'
    sequences = [seq for row_seq in table_sequences for seq in row_seq]
    write_unique_fasta(fname, ProteinAlignment(sequences))


def get_parser():
    """
    Create a command line parser.

    :return: the command line parser
    :rtype: argparse.ArgumentParser
    """

    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        '-cols',
        metavar='LETTERS',
        help=
        'A comma-separated list of column letters containing the FASTA strings,'
        ' e.g., "B,D".',
        required=True)
    parser.add_argument(
        '-rows',
        help='The rows containing the FASTA strings, e.g.,  "2:5".',
        required=True)
    parser.add_argument(
        '-name',
        default='A',
        metavar='LETTER',
        help='The column letter for the sequence name (default "%(default)s").')
    parser.add_argument(
        '-chain',
        metavar='INTEGER',
        default=1,
        help='The row number for the chain name(s) (default %(default)d).')
    parser.add_argument(
        '-separate',
        action='store_true',
        help='Create separate FASTA files (one per row). Default create single '
        'FASTA file using the base file name based on the input file.')
    parser.add_argument(
        '-sheet',
        help=
        'Workbook sheet to read. Default is the active worksheet. Use double '
        'quotes if the name has spaces or special characters in it, e.g., '
        '"FASTA sequences.xlsx".')
    parser.add_argument('input_file', help="Input xlsx file.")
    return parser


def main():
    parser = get_parser()
    opts = parser.parse_args()

    if not os.path.isfile(opts.input_file):
        parser.error(f'Input file {opts.input_file} not found.')

    try:
        start_idx, end_idx = opts.rows.split(':')
    except ValueError:
        parser.error('the -rows argument is not properly formatted.')
    try:
        opts.rows = list(range(int(start_idx), int(end_idx) + 1))
    except ValueError:
        parser.error('The -rows argument does not contain two integer values.')

    opts.cols = opts.cols.split(',')

    convert_to_fasta(opts)


if __name__ == '__main__':
    main()
